const system = require('../../../system');
const http = require('http');
const querystring = require('querystring');
const settings = require('../../../../config/settings');
const CtlBase = require('../../ctl.base');

class CompanyCtl extends CtlBase {
  constructor() {
    super('common', CtlBase.getServiceName(CompanyCtl));
    this.userSve = system.getObject('service.auth.userSve');
  }

  async update(p, q, req) {
    if (p.company_id != 1) {
      if (p.company_id != p.id) {
        throw new Error('没有权限');
      }
    }
    const u = await super.update(p, q, req);
    // 缓存失效
    await this.cacheManager.CompanyCache.invalidate(p.companykey);
    const company = await this.cacheManager.CompanyCache.cache(p.companykey);
    return system.getResult(company);
  }

  async getMyApps(p, q, req) {
    const userfind = await this.cacheManager.UserCache.cache(p.username);
    const { isSuper } = userfind;
    if (userfind.company.appids && userfind.company.appids != '') {
      const appsarray = userfind.company.appids.split(',');
      const appidsquery = appsarray.map(astr => astr.split('|')[0]);
      const apps = await this.service.getMyApps(appidsquery, isSuper);
      return system.getResult(apps);
    }
    return [];
  }

  async bindApps(p, q, req) {
    const { appids } = p;
    const cmpid = p.postcmpid;
    const appids2 = appids.map(item => `${item.appid}|${item.title}`);
    const appidstrs = appids2.join(',');
    await this.service.bindApps(appidstrs, cmpid);
    return system.getResult(appids);
  }

  async setOrgs(p, q, req) {
    // let companynew=await this.service.findById(p.company_id)
    const orgs = await this.service.setOrgs(p);
    return system.getResult(orgs);
  }

  async getOrgs(p, q, req) {
    // let companynew=await this.cacheManager["CompanyCache"].cache(req.xctx.fromcompanykey)
    const companynew = await this.service.findById(p.company_id);
    const userData = await this.userSve.findById(p.userid);
    const orgjsonstr = companynew.orgJson;
    let rtnjson = null;
    if (orgjsonstr && orgjsonstr != '') {
      rtnjson = JSON.parse(companynew.orgJson);
    } else {
      rtnjson = [];
    }
    if (userData) {
      this.service.formatOrgs(rtnjson);
      if (userData.isAdmin || userData.isSuper) {
        return system.getResult({ orgJson: rtnjson });
      }
      return system.getResult({ orgJson: this.service.buildOrgs(rtnjson, userData.ptags) });
    }
    return system.getResult({ orgJson: [] });
  }

  async getWatchOrgNodes(p, q, req) {
    const result = await this.service.getWatchOrgNodes(p.company_id);
    return result;
  }

  async refQuery(pobj, qobj, req) {
    const rtn = await this.service.refQuery(pobj);
    return rtn;
  }
}

module.exports = CompanyCtl;
