const system = require('../../../system');
const ServiceBase = require('../../sve.base');
const _ = require('lodash');
class ProductpriceService extends ServiceBase {
  constructor() {
    super('product', ServiceBase.getDaoName(ProductpriceService));
  }
  /**
   * @param {*} pname 产品名称
   * @param {*} spname 服务商名称
   */
  async findRegionsByProductName(productname, spname) {
    const ps = await this.dao.model.findAll({
      where: {
        sptags: { [this.db.Op.like]: `%${spname}%` },
        pname: { [this.db.Op.like]: `%${productname}%` },
        isEnabled: true,
      }, attributes: ['pname', 'strategyitems'], raw: true,
    });
    // 查询出地区
    const rs = await this.db.models.region.findAll({ attributes: ['code', 'name'], raw: true });
    // 获取地区JSON 对象
    const psrtn = [];
    const cacheregions = {};
    ps.forEach((r) => {
      const rpathstr = r.pname.split('~')[1];
      const rpatharray = rpathstr.split('/');
      const lstindex = rpatharray.length - 1;
      const regionName = rpatharray[lstindex];
      const regioninfo = {};
      regioninfo.label = regionName;
      const exAttrTmp = r.strategyitems;
      // 按照名字去取地区编码
      const areainfos = rs.filter((f) => {
        if (f.name.indexOf(regionName) >= 0) {
          return true;
        }
        return false;
      });
      let codeval = String(areainfos[0].code).padEnd(6, '0');
      if ((productname.indexOf('icp') < 0 && productname.indexOf('edi') < 0) && (areainfos[0].name.indexOf('天津') >= 0 || areainfos[0].name.indexOf('北京') >= 0 || areainfos[0].name.indexOf('上海') >= 0 || areainfos[0].name.indexOf('重庆') >= 0)) {
        codeval = String(`${areainfos[0].code}01`).padEnd(6, '0');
      }
      if (!cacheregions[codeval]) {
        cacheregions[codeval] = regioninfo;
        regioninfo.code = codeval;
        regioninfo.exAttr = [exAttrTmp];
        psrtn.push(regioninfo);
      } else {
        const cacheObj = cacheregions[codeval];
        cacheObj.exAttr.push(exAttrTmp);
      }
    });
    return psrtn;
  }

  /**
   * 为 财税 提供 产品 详情 接口
   * @param {*} productname  产品名称  小规模纳税人、一般纳税人
   * @param {*} region  地区
   * @param {*} spname  服务商
   */
  async findProductInfoForFt(productname, region, spname = '公司宝') {
    const datas = await this.dao.model.findAll({
      where: {
        $and: [
          {
            sptags: { $like: `%${spname}%` },
            isEnabled: true,
            pname: { $like: '%' + '财税服务' + '%' },
          }, {
            pname: { $like: '%' + '代理记账' + '%' },
          },
        ],
      },
      attributes: ['pname', 'strategyitems', 'hignpriceref'], // 价格、地区、skucode
      raw: true,
    });
    let result = [];
    // 查询出地区
    const address = await this.db.models.region.findAll({ attributes: ['code', 'name'], raw: true });
    for (const val of datas) {
      let area = val.pname.split('~')[1].split('/');
      area = area[area.length - 1];
      // 按照名字去取地区编码
      let areaCode = address.filter((f) => {
        if (f.name.indexOf(area) >= 0) {
          return true;
        }
        return false;
      });
      if (['北京市', '天津市', '上海市', '重庆市'].includes(area)) {
        areaCode[0].code = `${areaCode[0].code}01`;
      }
      areaCode = `${areaCode[0].code}00`;
      result.push({
        area,
        areaCode,
        type: val.strategyitems,
        price: val.hignpriceref,
      });
    }
    result = _.groupBy(result, 'type');

    if (productname) {
      result = result[productname];
    }
    if (productname && region) {
      result = _.find(result, item => item.area.indexOf(region) >= 0);
    }
    return result || [];
  }
  async updownProduct(uid) {
    const p = await this.dao.findById(uid);
    p.isEnabled = !p.isEnabled;
    await p.save();
    return {};
  }
}
module.exports = ProductpriceService;

// let p=new ProductpriceService()
// p.findRegionsByProductName("个体户注册","gao").then((res)=>{
//   console.log(res)
// })
